# 22年估值信息
import pandas as pd
from sqlalchemy import func, and_, literal
from sqlalchemy.orm import aliased, Session
from sdic.db.models import FCTValuationBase, VInvestProject
from datetime import datetime


def get_eva(session: Session, date_str: str) -> pd.DataFrame:
    fvb: FCTValuationBase = aliased(FCTValuationBase)
    vip: VInvestProject = aliased(VInvestProject)
    row_num = func.row_number().over(
        partition_by=[fvb.FD_PROJECT_ID, fvb.FD_FUND_ID],
        order_by=fvb.D_START.desc()
    ).label('rn')

    subq = (
        session.query(
            fvb.EVA.label("VALUATION_RESULT"),
            literal('估值').label("CAPITAL_TYPE"),
            fvb.D_START.label("FD_DATE"),
            vip.FD_SHOR_NAME,
            vip.VC_FUND_NAME,
            row_num,
        ).outerjoin(
            vip,
            and_(
                vip.FD_PROJECT_ID == fvb.FD_PROJECT_ID,
                vip.FD_FUND_ID == fvb.FD_FUND_ID
            )
        ).filter(
            vip.FD_GSTYPE == '国投创益',
            vip.VC_FUND_NAME != '创益盛屯基金',
            vip.FD_INVESTMENT_STATUS.in_([0, 2]),
            fvb.D_START <= datetime.strptime(date_str, '%Y-%m-%d').date(),
        ).subquery()
    )
    eva_rows = session.query(subq).filter(subq.c.rn == 1).all()
    eva_df = pd.DataFrame(eva_rows)
    return eva_df
